from pubtools.sign.operations import (
    ClearSignOperation,
    ContainerSignOperation,
)


def test_containersign_operation_doc_argument():
    assert ContainerSignOperation.doc_arguments() == {
        "options": {
            "digests": {"description": "List of digest to sign"},
            "references": {"description": "List of references to sign"},
            "signing_key": {"description": "Signing key short id which should be used for signing"},
            "task_id": {
                "description": "Usually pub task id, serves as identifier for in signing request"
            },
        },
        "examples": {"digests": "", "references": "", "signing_key": "", "task_id": ""},
    }


def test_clearsign_operation_doc_argument():
    assert ClearSignOperation.doc_arguments() == {
        "options": {
            "inputs": {"type": "list", "description": "Signing data", "required": "true"},
            "signing_key": {
                "type": "str",
                "description": "Signing key short id which should be used for signing",
                "required": "true",
            },
            "task_id": {
                "type": "str",
                "description": "Usually pub task id, serves as identifier for in signing request",
                "required": "true",
            },
        },
        "examples": {"inputs": ["input1", "input2"], "signing_key": "123", "task_id": "1"},
    }



from ..models.msg import MsgError

from proton.handlers import MessagingHandler


class _MsgClient(MessagingHandler):
    def __init__(self, errors):
        super().__init__()
        self.errors = errors

    def on_error(self, event, source=None):
        self.errors.append(
            MsgError(
                name=event,
                description=source.condition or source.remote_condition,
                source=source,
            )
        )
        event.container.stop()

    def on_link_error(self, event):
        self.on_error(event, event.link)

    def on_session_error(self, event):
        self.on_error(event, event.session)

    def on_connection_error(self, event):
        self.on_error(event, event.connection)

    def on_transport_error(self, event):
        self.on_error(event, event.transport)



from __future__ import annotations

from dataclasses import field, dataclass

from typing import List, ClassVar

from ..results.operation_result import OperationResult

from .base import SignOperation


@dataclass
class ContainerSignOperation(SignOperation):
    """ContainersSignOperation model class."""

    ResultType: ClassVar[OperationResult]
    digests: List[str] = field(metadata={"description": "List of digest to sign"})
    references: List[str] = field(metadata={"description": "List of references to sign"})
    signing_key: str = field(
        metadata={"description": "Signing key short id which should be used for signing"}
    )
    task_id: str = field(
        metadata={"description": "Usually pub task id, serves as identifier for in signing request"}
    )



from pubtools.sign.signers.msgsigner import (
    ContainerSignResult,
    ClearSignResult,
)


def test_containeroperation_result_to_dict():
    assert ContainerSignResult(signed_claims=["test"], signing_key="signing_key").to_dict() == {
        "signed_claims": ["test"],
        "signing_key": "signing_key",
    }


def test_clearoperation_result_to_dict():
    assert ClearSignResult(outputs=["test"], signing_key="signing_key").to_dict() == {
        "outputs": ["test"],
        "signing_key": "signing_key",
    }



import collections
import json
import socket
import logging
from multiprocessing import Process
import threading
import tempfile
import uuid

from .conftest_msgsig import f_msg_signer  # noqa: F401


from proton import Endpoint
from proton.reactor import Container
import proton
from proton import Message

from pytest import fixture

from pubtools.sign.clients.msg import _MsgClient


LOG = logging.getLogger("pubtools.sign.signers.radas")


class _Queue(object):
    def __init__(self, address, dynamic=False):
        self.address = address
        self.dynamic = dynamic
        self.queue = collections.deque()
        self.consumers = []

    def subscribe(self, consumer):
        LOG.debug("QUEUE SUBSCRIBED: %s %s", self.address, consumer)
        self.consumers.append(consumer)

    def unsubscribe(self, consumer):
        LOG.debug("QUEUE UNSUBSCRIBED: %s %s", self.address, consumer)
        if consumer in self.consumers:
            self.consumers.remove(consumer)
        return len(self.consumers) == 0 and (self.dynamic or len(self.queue) == 0)

    def publish(self, message):
        self.queue.append(message)
        self.dispatch()

    def dispatch(self, consumer=None):
        if consumer:
            c = [consumer]
        else:
            c = self.consumers
        LOG.debug("QUEUE DISPATCH TO %s %s", self.address, c)
        while self._deliver_to(c):
            pass

    def _deliver_to(self, consumers):
        try:
            result = False
            for c in consumers:
                if c.credit:
                    message = self.queue.popleft()
                    LOG.debug("QUEUE DELIVER TO %s %s", self.address, c, message)
                    c.send(message)
                    result = True
            return result
        except IndexError:  # no more messages
            LOG.debug("QUEUE NOTHING TO DELIVER %s", self.address)
            return False


class _Broker(_MsgClient):
    def __init__(self, url):
        super().__init__(errors=[])
        self.url = url
        self.queues = {}

    def on_start(self, event):
        LOG.debug("BROKER on start", self.url)
        self.acceptor = event.container.listen(self.url)

    def _queue(self, address):
        if address not in self.queues:
            self.queues[address] = _Queue(address)
        return self.queues[address]

    def on_link_opening(self, event):
        LOG.debug(
            "BROKER on_link_opening event",
            event.link,
            "source addr:",
            event.link.source.address,
            "remote source addr",
            event.link.remote_source.address,
            "target addr:",
            event.link.target.address,
            "remote target addr",
            event.link.remote_target.address,
        )
        if event.link.is_sender:
            if event.link.remote_source.dynamic:
                address = str(uuid.uuid4())
                event.link.source.address = address
                q = _Queue(address, True)
                self.queues[address] = q
                q.subscribe(event.link)
            elif event.link.remote_source.address:
                event.link.source.address = event.link.remote_source.address
                self._queue(event.link.source.address).subscribe(event.link)
        elif event.link.remote_target.address:
            event.link.target.address = event.link.remote_target.address

    def _unsubscribe(self, link):
        if link.source.address in self.queues and self.queues[link.source.address].unsubscribe(
            link
        ):
            del self.queues[link.source.address]

    def on_link_closing(self, event):
        LOG.debug(">> BROKER On link closing", event)
        if event.link.is_sender:
            self._unsubscribe(event.link)

    def on_disconnected(self, event):
        LOG.debug(">> BROKER On disconnected", event)
        self.remove_stale_consumers(event.connection)

    def remove_stale_consumers(self, connection):
        link = connection.link_head(Endpoint.REMOTE_ACTIVE)
        LOG.debug("BROKER removing stale consumer", link)
        while link:
            if link.is_sender:
                self._unsubscribe(link)
            link = link.next(Endpoint.REMOTE_ACTIVE)

    def on_sendable(self, event):
        LOG.debug("BROKER on_sendable", event.link.source.address)
        self._queue(event.link.source.address).dispatch(event.link)

    def on_message(self, event):
        LOG.debug("BROKER ON MESSAGE", event.message)
        address = event.link.target.address
        if address is None:
            address = event.message.address
        LOG.debug("BROKER publish", address)
        self._queue(address).publish(event.message)


class _BrokenBroker(_Broker):
    def on_sendable(self, event):
        LOG.debug("BROKER on_sendable", event.link.source.address)
        self._queue(event.link.source.address).dispatch(event.link)
        raise ValueError("Simulated broker error")
        event.on_link_error(event)


class _FakeMsgSigner(proton.handlers.MessagingHandler):
    def __init__(self, broker_urls, listen_to, send_to, cert, ca_cert, received_messages):
        self.broker_urls = broker_urls
        self.listen_to = listen_to
        self.send_to = send_to
        self.ssl_domain = proton.SSLDomain(proton.SSLDomain.MODE_CLIENT)
        self.ssl_domain.set_peer_authentication(proton.SSLDomain.ANONYMOUS_PEER)
        self.received_messages = received_messages
        super().__init__()
        self.to_send = []

    def on_start(self, event):
        conn = event.container.connect(
            urls=self.broker_urls, ssl_domain=self.ssl_domain, sasl_enabled=False
        )
        self.receiver = event.container.create_receiver(conn, self.listen_to)

    def on_message(self, event):
        LOG.debug("RADAS on message", event.message)
        headers = event.message.properties
        self.received_messages.append(event.message)

        if headers.get("mtype") == "container_signature":
            reply_message = ""
        if headers.get("mtype") == "clearsig_signature":
            reply_message = ""
        else:
            reply_message = event.message.body

        reply = Message()
        reply.address = self.send_to
        LOG.debug("Send to", self.send_to)
        reply.body = reply_message
        reply.properties = event.message.properties

        sender = event.container.create_sender(event.connection)
        LOG.debug("RADAS Sending", reply)
        sender.send(reply)
        LOG.debug("RADAS Sent")

    def on_sendable(self, event):
        LOG.debug("RADAS on sendable")
        if not self.to_send:
            LOG.debug("RADAS Nothing to send")
            return
        message_to_send = self.to_send.pop(0)
        event.sender.send(message_to_send)


class _StrayFakeMsgSigner(_FakeMsgSigner):
    def on_message(self, event):
        headers = event.message.properties
        self.received_messages.append(event.message)

        if headers.get("mtype") == "container_signature":
            reply_message = ""
        if headers.get("mtype") == "clearsig_signature":
            reply_message = ""
        else:
            reply_message = json.loads(event.message.body)

        reply = Message()
        reply.address = self.send_to
        reply_message["msg"]["request_id"] += "1"
        reply.body = json.dumps(reply_message)

        reply.properties = event.message.properties

        sender = event.container.create_sender(event.connection)
        sender.send(reply)


@fixture(scope="session")
def f_msgsigner_listen_to_topic():
    return "topic://Topic.pubtools.sign"


@fixture(scope="session")
def f_msgsigner_send_to_queue():
    return "topic://Topic.signatory.sign"


@fixture(scope="session")
def f_msgsigner_listen_to_topic_stray():
    return "topic://Topic.pubtools.sign.stray"


@fixture(scope="session")
def f_msgsigner_send_to_queue_stray():
    return "topic://Topic.signatory.sign.stray"


@fixture(scope="session")
def f_received_messages():
    return []


@fixture
def f_cleanup_msgsigner_messages(f_received_messages):
    return f_received_messages.clear()


@fixture(scope="session")
def f_find_available_port():
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.bind(("", 0))
    port = sock.getsockname()[1]
    sock.close()
    return port


@fixture(scope="session")
def f_find_available_port_for_broken():
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.bind(("", 0))
    port = sock.getsockname()[1]
    sock.close()
    return port


@fixture(scope="session")
def f_qpid_broker(f_find_available_port):
    LOG.debug("starting broker", f"localhost:{f_find_available_port}")
    broker = Container(_Broker(f"localhost:{f_find_available_port}"))
    p = Process(target=broker.run, args=())
    p.start()
    yield (broker, f_find_available_port)
    LOG.debug("destroying qpid broker")
    p.terminate()


@fixture(scope="session")
def f_broken_qpid_broker(f_find_available_port_for_broken):
    LOG.debug("starting broker", f"localhost:{f_find_available_port_for_broken}")
    broker = Container(_BrokenBroker(f"localhost:{f_find_available_port_for_broken}"))
    p = Process(target=broker.run, args=())
    p.start()
    yield (broker, f_find_available_port_for_broken)
    LOG.debug("destroying qpid broker")
    p.terminate()


@fixture(scope="session")
def f_fake_msgsigner(
    f_find_available_port,
    f_msgsigner_listen_to_topic,
    f_msgsigner_send_to_queue,
    f_received_messages,
):
    fr = _FakeMsgSigner(
        [f"localhost:{f_find_available_port}"],
        f_msgsigner_listen_to_topic,
        f_msgsigner_send_to_queue,
        "",
        "",
        f_received_messages,
    )
    frc = Container(fr)
    t = threading.Thread(target=frc.run, args=())
    t.start()
    yield fr, frc, f_received_messages
    frc.stop()


@fixture(scope="session")
def f_fake_msgsigner_stray(
    f_find_available_port,
    f_msgsigner_listen_to_topic_stray,
    f_msgsigner_send_to_queue_stray,
    f_received_messages,
):
    fr = _StrayFakeMsgSigner(
        [f"localhost:{f_find_available_port}"],
        f_msgsigner_listen_to_topic_stray,
        f_msgsigner_send_to_queue_stray,
        "",
        "",
        f_received_messages,
    )
    frc = Container(fr)
    t = threading.Thread(target=frc.run, args=())
    t.start()
    yield fr, frc, f_received_messages
    frc.stop()


@fixture
def f_client_certificate():
    with tempfile.NamedTemporaryFile() as tmpf:
        tmpf.write(
            """
-----BEGIN RSA PRIVATE KEY-----
MIIEpQIBAAKCAQEAxcdD7r4IdRBrylZnLNmHMu4AHnt1ReE7LhvIJj9XF+akW9pa
Nc7rY+dHGZVd45VUsS37MuGRzkl4gUs/wAh+4PQK+dDGqDY5nJL7QNgw96s1129d
O09inhUBFp7tMfc+xAuNvDSrxRHthwnDIU2Jrbl6fzube07g0qx7geCqxti/g5cg
xuQBQpHSpaRgZ7cyNuiyww1vssbqqAG8UZHb4SpwaqjXnPp6Vd/yrU3bRKAtD5s5
+ZxCa/dUJfsqS3jxQMlfH0lbvxZ5yR+4pLRAISKLfdTDcN56XJ6IstNyusNhr/76
C/CfWUN+oe5yCWbRx4L5Q6lcpDq5DcbMNswEoQIDAQABAoIBAQCUDQWCW0jzcNZv
wdw8S54Udusp5ls0c1Ucv/lFAFdO8f2JMNwkuX+l6oRj11dQPQIIHBaV0RuXo5IM
n2racsGf3a+1sB513xmjZreko/GMBOMqIRhWhKebFLga2d9PbvjSQp/YCkmnHTOE
yb8DWAq/PEBBrDpIxRQxQKK355mPfW/OT2ds/C9Qeg/JtqX8N+OyPLzXNV1Ga1OE
GSMm0sv4TTyq5MrQDxx3C/MkjEwnDWkjV5kQhQHGJ+ehlXbkQqIAyfOLvWLIhu7t
e9mWRykqoBJUU1/eks9QohHtrgFHdbwwI1bsxxZCTirRXPpen0/SEWZ4jUkPB6MV
vdqGxJoRAoGBAP92liL2yMP/27Bi/hBjTpKMCZsH7Hl04RSrJZ6AzxPlglh2deCG
bpmffItcvUymLKMbIixo2mdQ0M3tqIFtA7k7M/ZGRNkYRI17J4kXoFa0qvX4zULl
csLDau0DMVTrCoXRlWmozp9Ony4kAp5xh7ygIFyE+rw2G48pxNkotOkNAoGBAMYx
pnZQn1eD4TLERy+s679LWupFLjlasaYhHM4mOZR36ABIjzbUSTe5qG8qXs2V0UXF
G8vaGp8LLIwWg304WWTcErJru6qrp08OlHHPqytuDrQcpqwMkAgA4/xezGJW9ljJ
wYWTEonWMlfKFP04kVg+CqATBnjVMCii5SjAmrzlAoGBANF8f1WgpbYEZDTamJj7
tnz6FQ5qiwJ2U/TM/AZkfmtEc4Tzb2p1EtErNchafmkSg9wk7fsY6LB8Vx3nW5z2
tmz5HX1A1khoXB7g9OS42SUA9ojKRBgta9RGx7IgQh3uuCxQV4PTh8yffm0p3nPr
iXGmpaL48VvRyvu1NtUVSnUpAoGAeyWEigVkTItsFRAyLyRhwxW+YswjgY2hzljK
viiwJFkwtWRgYDAdYluglZodF96cDp7/u3VEj0fxIQYoI1ks6md30pbwH4bSyWOE
xwbDE5Qp3K3kvgh8QgzTnA8HLZ9dKCQMc8PDhBOsajHtQr2wScUa8wV/QvssFkPI
4b5zJyUCgYEAra20VWtCDvtgVkEYEV3Taqp9lx8g4sW+E3O0/q5hUKjmyC1+xjwS
i4CYcqbOwSpEBJ6SInJ8qO9mr5H34IrNVMDqQFtQeUs14BubD5SL789jPzzYlPuf
WtgtGGUuxENK+DcU7zn6px9PqS0Rd8paC7CWrVZpn9fLusC8DAYNyG8=
-----END RSA PRIVATE KEY-----
-----BEGIN CERTIFICATE-----
MIIDvjCCAqagAwIBAgIUTTW9wgsEcsPa+nC9LNIY2NQHs/gwDQYJKoZIhvcNAQEL
BQAwazELMAkGA1UEBhMCdVMxFTATBgNVBAcMDERlZmF1bHQgQ2l0eTEQMA4GA1UE
CgwHUmVkIEhhdDEbMBkGA1UECwwSQ2xvdWQgRGlzdHJpYnV0aW9uMRYwFAYDVQQD
DA1wdWJ0b29scy1zaWduMB4XDTIzMDUyNTEyNDQwNVoXDTI1MDgyNzEyNDQwNVow
cDELMAkGA1UEBhMCVVMxFTATBgNVBAcMDERlZmF1bHQgQ2l0eTEQMA4GA1UECgwH
UmVkIEhhdDEbMBkGA1UECwwSQ2xvdWQgRGlzdHJpYnV0aW9uMRswGQYDVQQDDBJw
dWJ0b29scy1zaWduLXRlc3QwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIB
AQDFx0Puvgh1EGvKVmcs2Ycy7gAee3VF4TsuG8gmP1cX5qRb2lo1zutj50cZlV3j
lVSxLfsy4ZHOSXiBSz/ACH7g9Ar50MaoNjmckvtA2DD3qzXXb107T2KeFQEWnu0x
9z7EC428NKvFEe2HCcMhTYmtuXp/O5t7TuDSrHuB4KrG2L+DlyDG5AFCkdKlpGBn
tzI26LLDDW+yxuqoAbxRkdvhKnBqqNec+npV3/KtTdtEoC0Pmzn5nEJr91Ql+ypL
ePFAyV8fSVu/FnnJH7iktEAhIot91MNw3npcnoiy03K6w2Gv/voL8J9ZQ36h7nIJ
ZtHHgvlDqVykOrkNxsw2zAShAgMBAAGjVTBTMB8GA1UdIwQYMBaAFA/iBvXjjDuu
XOQQoMnHcOtLe896MAkGA1UdEwQCMAAwCwYDVR0PBAQDAgTwMBgGA1UdEQQRMA+C
DWhlbGxmaXNoLnRlc3QwDQYJKoZIhvcNAQELBQADggEBAMpT7zdPkO4gb3tn74Nb
aFIhRDr4sQorpMlxRYQZSKV5qjvZ364bHaig380iQMwVs2H09z6IpO/8eXbrtlaD
z3QH2bt+yofk+JbWjQky7fKB1ZXvpugcNMTgr2OuupqPIkHaE4N+lZmaasMDY3+i
+1F5U/0EaNYuFt7Tv6uS9cqkbUuvm9Elm71arkDSrdOycTXGmm+DB8XnAGnQpNWQ
L3WmvVmWzvgjYOxvef2nNE3SeM3+/ZDCNYIJGC3KS/pOdshF95Xy00mkXv+Z0wAL
iuT88ZskJ2WDrduvxxKPSFUZ1ncZMsRAcNp8B0+JeR9Q30TGJCMNqJ3jg4u8Pvod
lq4=
-----END CERTIFICATE-----
""".encode(
                "utf-8"
            )
        )
        tmpf.flush()
        yield tmpf.name


@fixture
def f_ca_certificate():
    with tempfile.NamedTemporaryFile() as tmpf:
        tmpf.write(
            """
-----BEGIN CERTIFICATE-----
MIIDtzCCAp+gAwIBAgIUATUd1WliG6ETZqKP8EZyijG9xUIwDQYJKoZIhvcNAQEL
BQAwazELMAkGA1UEBhMCdVMxFTATBgNVBAcMDERlZmF1bHQgQ2l0eTEQMA4GA1UE
CgwHUmVkIEhhdDEbMBkGA1UECwwSQ2xvdWQgRGlzdHJpYnV0aW9uMRYwFAYDVQQD
DA1wdWJ0b29scy1zaWduMB4XDTIzMDUyNTEyNDQwMloXDTI4MDUyMzEyNDQwMlow
azELMAkGA1UEBhMCdVMxFTATBgNVBAcMDERlZmF1bHQgQ2l0eTEQMA4GA1UECgwH
UmVkIEhhdDEbMBkGA1UECwwSQ2xvdWQgRGlzdHJpYnV0aW9uMRYwFAYDVQQDDA1w
dWJ0b29scy1zaWduMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA2e8m
tgCPFummVobuc+IHdTHfdDVYnMYyp3+ZRrzc5yCYjRpeIoABzcI/aZza2kljHNKZ
yan6LnsdD45xuyhDVmgMHij7Luq0p8ibQAQvmf6kh1pMgFy3Mtsm+lwT99Bt8gNN
YUiahSkN+vZa3eswZTzu5z5RkztzZt4O9qzsdUR7tKPjB5OlvsZFyFnvgtnAByqh
bLpe/YHR/A79TWgzZFBt67/f4ghGHUtN+CPB6e+TLKQ9QRsOqZLhuMwlNsJSBc8k
duvaXYDU3w+GXMci7pWIk3HM2Z9m0AxizZe8ygz/wworSxC8CS2WwPil3W9ft4iT
LTxBoEtN2MQUTMyo9QIDAQABo1MwUTAdBgNVHQ4EFgQUD+IG9eOMO65c5BCgycdw
60t7z3owHwYDVR0jBBgwFoAUD+IG9eOMO65c5BCgycdw60t7z3owDwYDVR0TAQH/
BAUwAwEB/zANBgkqhkiG9w0BAQsFAAOCAQEAsX9O5pnlmmXfm6Vx98bOp8o79g40
KTfi1KZg8M3wfWYGDhSpDqdJC/1IPdEWp8VWF68zymTVa5unXRPewwQ07SP5yn19
YFlQF7l9vSnVVt4/JRPB+ydBgSXoxK6b5zbEK8+3iqBuRGvp8u0rrn4ohEkserd+
tcKssr4IEdgeVNco+UStQrrIrf+KoPN147fKzwkaUZKj3ybVExHnilr4D+HB94jL
pH404Fud+v2NWjl7RSQnsMw+gCz6Sm3eU/aWC5L5ZOecawj01Qr60nv97eqc8tdG
TrXd8yRh0cI5wL5KnO4hL/kYwOOaKsMwEkNlmL2Io7DrhVgJUAWycqfHfA==
-----END CERTIFICATE-----""".encode(
                "utf-8"
            )
        )
        tmpf.flush()
        yield tmpf.name


@fixture
def f_config_msg_signer_ok(f_client_certificate):
    with tempfile.NamedTemporaryFile() as tmpf:
        tmpf.write(
            f"""
msg_signer:
  messaging_brokers:
    - amqps://broker-01:5671
    - amqps://broker-02:5671
  messaging_cert_key: {f_client_certificate}
  messaging_ca_cert: ~/messaging/ca-cert.crt
  topic_send_to: topic://Topic.sign
  topic_listen_to: queue://Consumer.{{creator}}.{{task_id}}.Topic.sign.{{task_id}}
  environment: prod
  service: pubtools-sign
  timeout: 1
  retries: 3
  message_id_key: request_id
  log_level: debug
        """.encode(
                "utf-8"
            )
        )
        tmpf.flush()
        yield tmpf.name


@fixture
def f_config_msg_signer_missing():
    with tempfile.NamedTemporaryFile() as tmpf:
        tmpf.write(
            f"""
msg_signer:
  messaging_brokers:
    - amqps://broker-01:5671
    - amqps://broker-02:5671
  messaging_cert_key: {f_client_certificate}
  messaging_ca_cert: ~/messaging/ca-cert.crt
  topic_listen_to: queue://Consumer.{{creator}}.{{task_id}}.Topic.sign.{{task_id}}
  environment: prod
  service: pubtools-sign
  timeout: 1
  retries: 3
  message_id_key: request_id
  log_level: debug""".encode(
                "utf-8"
            )
        )
        tmpf.flush()
        yield tmpf.name



import pytest
import piny

from pubtools.sign.conf.conf import load_config


def test_load_config_radas_ok(f_config_msg_signer_ok, f_client_certificate):
    assert load_config(f_config_msg_signer_ok) == {
        "msg_signer": {
            "messaging_brokers": ["amqps://broker-01:5671", "amqps://broker-02:5671"],
            "messaging_cert_key": f_client_certificate,
            "messaging_ca_cert": "~/messaging/ca-cert.crt",
            "topic_send_to": "topic://Topic.sign",
            "topic_listen_to": "queue://Consumer.{creator}.{task_id}.Topic.sign.{task_id}",
            "environment": "prod",
            "service": "pubtools-sign",
            "timeout": 1,
            "retries": 3,
            "message_id_key": "request_id",
            "log_level": "debug",
        }
    }


def test_load_config_missing(f_config_msg_signer_missing):
    with pytest.raises(piny.errors.ValidationError):
        assert load_config(f_config_msg_signer_missing)



import datetime


def set_log_level(logger, level):
    """Set log level for provided logger.

    :param logger: logger
    :type logger: logging.Logger
    :param level: logging level
    :type level: str
    """
    if level.upper() not in ("DEBUG", "INFO", "WARNING", "ERROR"):
        raise ValueError(f"Unknown log level {level}")
    logger.setLevel(level.upper())


def isodate_now():
    """Return current datetime in ISO-8601.

    :return: str
    """
    return datetime.datetime.utcnow().isoformat() + "Z"



import logging

import pytest

from pubtools.sign.utils import set_log_level


def test_set_log_level():
    LOG = logging.getLogger()
    set_log_level(LOG, "DEBUG")
    assert LOG.level == logging.DEBUG
    set_log_level(LOG, "INFO")
    assert LOG.level == logging.INFO
    set_log_level(LOG, "WARNING")
    assert LOG.level == logging.WARNING
    set_log_level(LOG, "ERROR")
    assert LOG.level == logging.ERROR
    with pytest.raises(ValueError):
        set_log_level(LOG, "UNKNOWN")



import dataclasses
from typing import Dict, Any


@dataclasses.dataclass
class MsgMessage:
    """Messaging message model."""

    headers: Dict[str, Any]
    address: str
    body: Dict[str, Any]


@dataclasses.dataclass
class MsgError:
    """Messaging error model."""

    name: str
    description: str
    source: Any



import json
import logging
from typing import List

from ..models.msg import MsgMessage, MsgError

from .msg import _MsgClient

import proton
import proton.utils
from proton.reactor import Container

LOG = logging.getLogger("pubtools.sign.signers.radas")


class _SendClient(_MsgClient):
    def __init__(
        self,
        messages: List[MsgMessage],
        broker_urls: List[str],
        cert: str,
        ca_cert: str,
        errors: List[MsgError],
    ):
        super().__init__(errors=errors)
        self.broker_urls = broker_urls
        self.messages = messages
        self.ssl_domain = proton.SSLDomain(proton.SSLDomain.MODE_CLIENT)
        if cert:
            self.ssl_domain.set_credentials(cert, cert, None)
        if ca_cert:
            self.ssl_domain.set_trusted_ca_db(ca_cert)
        self.ssl_domain.set_peer_authentication(proton.SSLDomain.ANONYMOUS_PEER)
        self.sent = 0
        self.confirmed = 0
        self.total = len(messages)

    def on_start(self, event):
        conn = event.container.connect(
            urls=self.broker_urls, ssl_domain=self.ssl_domain, sasl_enabled=False
        )
        self.sender = event.container.create_sender(conn)

    def on_sendable(self, event):
        LOG.debug("Sender on_sendable")
        if self.sent < self.total:
            message = self.messages[self.sent]
            LOG.debug("Sending message: %s %s %s", message.body, message.address, message.headers)
            event.sender.send(
                proton.Message(
                    properties=message.headers,
                    address=message.address,
                    body=json.dumps(message.body),
                )
            )
            self.sent += 1

    def on_accepted(self, event):
        LOG.debug("Sender accepted")
        self.confirmed += 1
        if self.confirmed == self.total:
            LOG.debug("Sender closing")
            event.connection.close()

    def on_disconnected(self, event):  # pragma: no cover
        self.sent = self.confirmed  # pragma: no cover


class SendClient(Container):
    """SendClient wrapper class."""

    def __init__(
        self,
        messages: List[MsgMessage],
        broker_urls: List[str],
        cert: str,
        ca_cert: str,
        retries: int,
        errors: List[MsgError],
    ):
        """Send Client Initializer.

        :param messages: List of messages to send.
        :type messages: List[MsgMessage]
        :param broker_urls: List of addresses of messaging broker
        :type messages: List[str]
        :param cert: Messaging client certificate
        :type cert: str
        :param retries: Number of retries for sending messages
        :type retries: int
        :param errors: List of errors which occured during the process
        :type errors: List[MsgError]
        """
        self.messages = messages
        self.handler = _SendClient(
            messages=messages, broker_urls=broker_urls, cert=cert, ca_cert=ca_cert, errors=errors
        )
        self._retries = retries
        self._errors = errors
        super().__init__(self.handler)

    def run(self):
        """Run the SendClient."""
        errors_len = 0
        if not len(self.messages):
            LOG.warning("No messages to send")
            return []
        for x in range(self._retries):
            super().run()
            if len(self._errors) == errors_len:
                break
            errors_len = len(self._errors)
        else:
            return self._errors
        return []



import marshmallow as ma
from piny import MarshmallowValidator, StrictMatcher, YamlLoader

CONFIG_PATHS = ["~/.config/pubtools-sign/conf.yaml", "/etc/pubtools-sign/conf.yaml"]


class MsgSignerSchema(ma.Schema):
    """Radas signer configuration schema."""

    messaging_brokers = ma.fields.List(ma.fields.String(), required=True)
    messaging_cert_key = ma.fields.String(required=True)
    messaging_ca_cert = ma.fields.String(required=True)
    topic_send_to = ma.fields.String(required=True)
    topic_listen_to = ma.fields.String(required=True)
    environment = ma.fields.String(required=True)
    service = ma.fields.String(required=True)
    timeout = ma.fields.Integer(required=True)
    retries = ma.fields.Integer(required=True)
    message_id_key = ma.fields.String(required=True)
    log_level = ma.fields.String(default="INFO")


class ConfigSchema(ma.Schema):
    """pubtools-sign configuration schema."""

    msg_signer = ma.fields.Nested(MsgSignerSchema)


def load_config(fname: str):
    """Load configuration from a filename.

    :param fname: filename
    :type fname: str

    :return Dict[str, Any]:
    """
    config = YamlLoader(
        path=fname,
        matcher=StrictMatcher,
        validator=MarshmallowValidator,
        schema=ConfigSchema,
    ).load(many=False)
    return config



from __future__ import annotations

from abc import ABC, abstractmethod
import dataclasses


@dataclasses.dataclass()
class OperationResult(ABC):
    """OperationResult abstract class."""

    @abstractmethod
    def to_dict(self: OperationResult):
        """Return dict representation of OperationResult."""
        ...  # pragma: no cover



from __future__ import annotations

from abc import ABC

from dataclasses import dataclass
from typing import ClassVar, Dict, Any

from ..results.operation_result import OperationResult


@dataclass
class SignOperation(ABC):
    """SignOperation Abstract class."""

    ResultType: ClassVar[OperationResult]

    @classmethod
    def doc_arguments(cls: SignOperation) -> Dict[str, Any]:
        """Return dictionary with arguments description of the operation."""
        doc_arguments = {}
        options_arguments_doc = {}
        exmaple_arguments_doc = {}

        for fn, fv in cls.__dataclass_fields__.items():
            if fv.metadata.get("description"):
                options_arguments_doc[fn] = {
                    field: fv.metadata[field] for field in fv.metadata if field != "sample"
                }
                exmaple_arguments_doc[fn] = fv.metadata.get("sample", "")
        doc_arguments["options"] = options_arguments_doc
        doc_arguments["examples"] = exmaple_arguments_doc

        return doc_arguments



from unittest.mock import patch

from pytest import fixture


@fixture
def f_msg_signer(f_config_msg_signer_ok):
    with patch("pubtools.sign.signers.msgsigner.MsgSigner") as msgsigner:
        yield msgsigner



from click.testing import CliRunner

from pubtools.sign.bundle import cli


def test_bundle_msg_container_sign(f_msg_signer, f_config_msg_signer_ok):
    f_msg_signer.return_value.sign.return_value.signer_results.to_dict.return_value = {
        "status": "ok"
    }
    f_msg_signer.return_value.sign.return_value.operation_result.signed_claims = []
    f_msg_signer.return_value.sign.return_value.operation_result.signing_key = ""
    result = CliRunner().invoke(
        cli,
        [
            "msg-container-sign",
            "--signing-key",
            "test-signing-key",
            "--digest",
            "some-digest",
            "--reference",
            "some-reference",
            "--task-id",
            "1",
            "--config",
            f_config_msg_signer_ok,
        ],
    )
    assert result.exit_code == 0, result.output


def test_bundle_msg_clear_sign(f_msg_signer, f_config_msg_signer_ok):
    f_msg_signer.return_value.sign.return_value.signer_results.to_dict.return_value = {
        "status": "ok"
    }
    f_msg_signer.return_value.sign.return_value.operation_result.outputs = []
    f_msg_signer.return_value.sign.return_value.operation_result.signing_key = ""
    result = CliRunner().invoke(
        cli,
        [
            "msg-clear-sign",
            "--signing-key",
            "test-signing-key",
            "--task-id",
            "1",
            "--config",
            f_config_msg_signer_ok,
            "hello world",
        ],
    )
    assert result.exit_code == 0, result.output



from __future__ import annotations

from dataclasses import field, dataclass

from typing import List, ClassVar

from ..results.operation_result import OperationResult

from .base import SignOperation


@dataclass
class ClearSignOperation(SignOperation):
    """ClearsSignOperation model class."""

    ResultType: ClassVar[OperationResult]
    inputs: List[str] = field(
        metadata={
            "type": "list",
            "description": "Signing data",
            "required": "true",
            "sample": ["input1", "input2"],
        }
    )
    signing_key: str = field(
        metadata={
            "type": "str",
            "description": "Signing key short id which should be used for signing",
            "required": "true",
            "sample": "123",
        }
    )
    task_id: str = field(
        metadata={
            "type": "str",
            "description": "Usually pub task id, serves as identifier for in signing request",
            "required": "true",
            "sample": "1",
        }
    )



import json
from unittest.mock import patch

import pytest
from ansible.module_utils import basic
from ansible.module_utils.common.text.converters import to_bytes

from pubtools.sign.ansible import msg_clear_sign
from pubtools.sign.conf.conf import load_config
from pubtools.sign.operations import ClearSignOperation
from pubtools.sign.results import ClearSignResult
from pubtools.sign.results.signing_results import SigningResults
from pubtools.sign.signers.msgsigner import MsgSigner, MsgSignerResults


def set_module_args(args):
    """Prepare arguments so that they will be picked up during module creation.

    :param args: ansible module input args
    :type args: dict
    """
    args = json.dumps({"ANSIBLE_MODULE_ARGS": args})
    basic._ANSIBLE_ARGS = to_bytes(args)


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case."""

    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case."""

    pass


def exit_json(*args, **kwargs):
    """Patch over exit_json, package return data into an exception."""
    if "changed" not in kwargs:
        kwargs["changed"] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):
    """Patch over fail_json, package return data into an exception."""
    kwargs["failed"] = True
    raise AnsibleFailJson(kwargs)


@patch.multiple(basic.AnsibleModule, exit_json=exit_json)
@patch("pubtools.sign.signers.msgsigner.MsgSigner.clear_sign")
def test_run_module_ok(clear_sign_mock, f_config_msg_signer_ok):
    signer = MsgSigner()
    signer.load_config(load_config(f_config_msg_signer_ok))

    parameters = {
        "inputs": ["hello world"],
        "signing_key": "test-signing-key",
        "task_id": "1",
        "config": f_config_msg_signer_ok,
    }

    clear_sign_operation = ClearSignOperation(
        inputs=parameters["inputs"],
        signing_key=parameters["signing_key"],
        task_id="1",
    )

    clear_sign_mock.return_value = SigningResults(
        signer=signer,
        operation=clear_sign_operation,
        signer_results=MsgSignerResults(status="ok", error_message=""),
        operation_result=ClearSignResult(
            outputs=["signed:'hello world'"], signing_key="test-signing-key"
        ),
    )

    set_module_args(parameters)
    with pytest.raises(AnsibleExitJson) as result:
        msg_clear_sign.run_module()
    assert result.value.args[0]["changed"]
    assert (
        result.value.args[0]["message"]["signer_result"]
        == clear_sign_mock.return_value.signer_results.to_dict()
    )
    assert (
        result.value.args[0]["message"]["operation_results"]
        == clear_sign_mock.return_value.operation_result.outputs
    )
    assert (
        result.value.args[0]["message"]["signing_key"]
        == clear_sign_mock.return_value.operation_result.signing_key
    )


@patch.multiple(basic.AnsibleModule, fail_json=fail_json)
@patch("pubtools.sign.signers.msgsigner.MsgSigner.clear_sign")
def test_run_module_failed(clear_sign_mock, f_config_msg_signer_ok):
    signer = MsgSigner()
    signer.load_config(load_config(f_config_msg_signer_ok))

    parameters = {
        "inputs": [f"@{f_config_msg_signer_ok}"],
        "signing_key": "test-signing-key",
        "task_id": "1",
        "config": f_config_msg_signer_ok,
    }

    clear_sign_operation = ClearSignOperation(
        inputs=parameters["inputs"],
        signing_key=parameters["signing_key"],
        task_id="1",
    )

    clear_sign_mock.return_value = SigningResults(
        signer=signer,
        operation=clear_sign_operation,
        signer_results=MsgSignerResults(status="failed", error_message=""),
        operation_result=ClearSignResult(
            outputs=["signed:'hello world'"], signing_key="test-signing-key"
        ),
    )

    set_module_args(parameters)
    with pytest.raises(AnsibleFailJson) as result:
        msg_clear_sign.run_module()
    assert result.value.args[0]["failed"]


@patch.multiple(basic.AnsibleModule, fail_json=fail_json)
@patch("pubtools.sign.signers.msgsigner.MsgSigner.sign")
def test_run_module_exception(sign_mock, f_config_msg_signer_ok):
    sign_mock.side_effect = [ValueError("No configuration file found ...")]
    parameters = {
        "inputs": ["hello world"],
        "signing_key": "test-signing-key",
        "task_id": "1",
        "config": f_config_msg_signer_ok,
    }
    set_module_args(parameters)
    with pytest.raises(AnsibleFailJson) as result:
        msg_clear_sign.run_module()
    assert result.value.args[0]["failed"]


@patch.multiple(basic.AnsibleModule, exit_json=exit_json)
@patch("pubtools.sign.signers.msgsigner.MsgSigner.sign")
def test_run_module_check_mode(sign_mock, f_config_msg_signer_ok):
    parameters = {
        "inputs": ["hello world"],
        "signing_key": "test-signing-key",
        "task_id": "1",
        "config": f_config_msg_signer_ok,
        "_ansible_check_mode": True,
    }
    set_module_args(parameters)
    with pytest.raises(AnsibleExitJson):
        msg_clear_sign.run_module()



from __future__ import annotations
import dataclasses

from typing import List, ClassVar
from typing_extensions import Self

from ..results.operation_result import OperationResult


@dataclasses.dataclass
class ContainerSignResult(OperationResult):
    """ContainerOperationResult model."""

    ResultType: ClassVar[OperationResult]
    signed_claims: List[str]
    signing_key: str

    def to_dict(self: Self):
        """Return dict representation of ContainerOperationResult."""
        return {"signed_claims": self.signed_claims, "signing_key": self.signing_key}



from unittest.mock import patch

from pubtools.sign.clients.msg_send_client import SendClient, _SendClient
from pubtools.sign.models.msg import MsgMessage

import json


def test_send_client_zero_messages(
    f_cleanup_msgsigner_messages,
    f_qpid_broker,
    f_fake_msgsigner,
    f_client_certificate,
    f_ca_certificate,
):
    qpid_broker, port = f_qpid_broker
    sc = SendClient([], [f"localhost:{port}"], f_client_certificate, f_ca_certificate, 10, [])
    sc.run()
    msgsigner, _, received_messages = f_fake_msgsigner
    assert [x.body for x in msgsigner.received_messages] == []


def test_send_client_send_message(
    f_cleanup_msgsigner_messages,
    f_qpid_broker,
    f_msgsigner_listen_to_topic,
    f_fake_msgsigner,
    f_client_certificate,
    f_ca_certificate,
):
    qpid_broker, port = f_qpid_broker
    message = MsgMessage(
        headers={}, address=f_msgsigner_listen_to_topic, body={"message": "test_message"}
    )
    sc = SendClient(
        [message], [f"localhost:{port}"], f_client_certificate, f_ca_certificate, 10, []
    )
    assert sc.run() == []
    msgsigner, _, received_messages = f_fake_msgsigner
    assert len(received_messages) == 1
    assert [x.body for x in received_messages] == [json.dumps(message.body)]


def test_send_client_errors(
    f_cleanup_msgsigner_messages,
    f_qpid_broker,
    f_msgsigner_listen_to_topic,
    f_fake_msgsigner,
    f_client_certificate,
    f_ca_certificate,
):
    qpid_broker, port = f_qpid_broker
    message1 = MsgMessage(
        headers={}, address=f_msgsigner_listen_to_topic, body={"message": "test_message1"}
    )

    on_start_original = _SendClient.on_start
    with patch(
        "pubtools.sign.clients.msg_send_client._SendClient.on_start", autospec=True
    ) as patched_on_accepted:
        errors = ["errors"]
        patched_on_accepted.side_effect = lambda self, event: [
            self.errors.append("1"),
            on_start_original(self, event),
        ]
        sc = SendClient(
            [message1], [f"localhost:{port}"], f_client_certificate, f_ca_certificate, 1, errors
        )
        assert sc.run() == ["errors", "1"]



import base64
import json
import os

from click.testing import CliRunner
import pytest
from unittest.mock import patch, Mock, ANY


from pubtools.sign.models.msg import MsgError
from pubtools.sign.signers.msgsigner import (
    MsgSigner,
    MsgSignerResults,
    msg_clear_sign,
    ContainerSignOperation,
    ContainerSignResult,
    ClearSignOperation,
    ClearSignResult,
    SignRequestType,
    msg_clear_sign_main,
    msg_container_sign_main,
    _get_config_file,
)
from pubtools.sign.conf.conf import load_config
from pubtools.sign.models.msg import MsgMessage
from pubtools.sign.exceptions import UnsupportedOperation
from pubtools.sign.results.signing_results import SigningResults


def test_msg_container_sign(f_msg_signer, f_config_msg_signer_ok):
    f_msg_signer.return_value.sign.return_value.signer_results.to_dict.return_value = {
        "status": "ok"
    }
    f_msg_signer.return_value.sign.return_value.operation_result.signed_claims = []
    f_msg_signer.return_value.sign.return_value.operation_result.signing_key = ""
    result = CliRunner().invoke(
        msg_container_sign_main,
        [
            "--signing-key",
            "test-signing-key",
            "--digest",
            "some-digest",
            "--reference",
            "some-reference",
            "--task-id",
            "1",
            "--config",
            f_config_msg_signer_ok,
        ],
    )
    print(result.stdout)
    assert result.exit_code == 0, result.output


def test_msg_container_sign_error(f_msg_signer, f_config_msg_signer_ok):
    f_msg_signer.return_value.sign.return_value.signer_results.to_dict.return_value = {
        "status": "error",
        "error_message": "simulated error",
    }
    f_msg_signer.return_value.sign.return_value.operation_result.signed_claims = []
    f_msg_signer.return_value.sign.return_value.operation_result.signing_key = ""
    result = CliRunner().invoke(
        msg_container_sign_main,
        [
            "--signing-key",
            "test-signing-key",
            "--digest",
            "some-digest",
            "--reference",
            "some-reference",
            "--task-id",
            "1",
            "--config",
            f_config_msg_signer_ok,
        ],
    )
    print(result.stdout)
    assert result.exit_code == 1, result.output


def test_msg_container_sign_raw(f_msg_signer, f_config_msg_signer_ok):
    f_msg_signer.return_value.sign.return_value.signer_results.to_dict.return_value = {
        "status": "ok"
    }
    f_msg_signer.return_value.sign.return_value.operation_result.signed_claims = ["signed"]
    f_msg_signer.return_value.sign.return_value.operation_result.signing_key = ""
    result = CliRunner().invoke(
        msg_container_sign_main,
        [
            "--signing-key",
            "test-signing-key",
            "--digest",
            "some-digest",
            "--reference",
            "some-reference",
            "--task-id",
            "1",
            "--config",
            f_config_msg_signer_ok,
            "--raw",
        ],
    )
    print(result.stdout)
    assert result.exit_code == 0, result.output
    assert result.output == "signed\n"


def test_msg_container_sign_raw_error(f_msg_signer, f_config_msg_signer_ok):
    f_msg_signer.return_value.sign.return_value.signer_results.to_dict.return_value = {
        "status": "error",
        "error_message": "simulated error",
    }
    f_msg_signer.return_value.sign.return_value.operation_result.signed_claims = []
    f_msg_signer.return_value.sign.return_value.operation_result.signing_key = ""
    result = CliRunner().invoke(
        msg_container_sign_main,
        [
            "--signing-key",
            "test-signing-key",
            "--digest",
            "some-digest",
            "--reference",
            "some-reference",
            "--task-id",
            "1",
            "--config",
            f_config_msg_signer_ok,
            "--raw",
        ],
    )
    print(result.stdout)
    assert result.exit_code == 1, result.output
    assert result.output == "simulated error\n"


def test_msg_clearsign_sign(f_msg_signer, f_config_msg_signer_ok):
    f_msg_signer.return_value.sign.return_value.signer_results.to_dict.return_value = {
        "status": "ok"
    }
    f_msg_signer.return_value.sign.return_value.operation_result.outputs = []
    f_msg_signer.return_value.sign.return_value.operation_result.signing_key = ""
    result = CliRunner().invoke(
        msg_clear_sign_main,
        [
            "--signing-key",
            "test-signing-key",
            "--task-id",
            "1",
            "--config",
            f_config_msg_signer_ok,
            "hello world",
        ],
    )
    assert result.exit_code == 0, result.output


def test_msg_clearsign_sign_error(f_msg_signer, f_config_msg_signer_ok):
    f_msg_signer.return_value.sign.return_value.signer_results.to_dict.return_value = {
        "status": "error",
        "error_message": "simulated error",
    }
    f_msg_signer.return_value.sign.return_value.operation_result.outputs = []
    f_msg_signer.return_value.sign.return_value.operation_result.signing_key = ""
    result = CliRunner().invoke(
        msg_clear_sign_main,
        [
            "--signing-key",
            "test-signing-key",
            "--task-id",
            "1",
            "--config",
            f_config_msg_signer_ok,
            "hello world",
        ],
    )
    assert result.exit_code == 1, result.output


def test_msg_clearsign_sign_raw(f_msg_signer, f_config_msg_signer_ok):
    f_msg_signer.return_value.sign.return_value.signer_results.to_dict.return_value = {
        "status": "ok"
    }
    f_msg_signer.return_value.sign.return_value.operation_result.outputs = ["signed"]
    f_msg_signer.return_value.sign.return_value.operation_result.signing_key = ""
    result = CliRunner().invoke(
        msg_clear_sign_main,
        [
            "--signing-key",
            "test-signing-key",
            "--task-id",
            "1",
            "--config",
            f_config_msg_signer_ok,
            "--raw",
            "hello world",
        ],
    )
    assert result.exit_code == 0, result.output
    assert result.output == "signed\n"


def test_msg_clearsign_sign_raw_error(f_msg_signer, f_config_msg_signer_ok):
    f_msg_signer.return_value.sign.return_value.signer_results.to_dict.return_value = {
        "status": "error",
        "error_message": "simulated error",
    }
    f_msg_signer.return_value.sign.return_value.operation_result.outputs = []
    f_msg_signer.return_value.sign.return_value.operation_result.signing_key = ""
    result = CliRunner().invoke(
        msg_clear_sign_main,
        [
            "--signing-key",
            "test-signing-key",
            "--task-id",
            "1",
            "--config",
            f_config_msg_signer_ok,
            "--raw",
            "hello world",
        ],
    )
    print(result.stdout)
    assert result.exit_code == 1, result.output
    assert result.output == "simulated error\n"


def test_msg_clearsign_sign_file_input(f_msg_signer, f_config_msg_signer_ok):
    f_msg_signer.return_value.sign.return_value.signer_results.to_dict.return_value = {
        "status": "ok"
    }
    f_msg_signer.return_value.sign.return_value.operation_result.outputs = []
    f_msg_signer.return_value.sign.return_value.operation_result.signing_key = ""
    result = CliRunner().invoke(
        msg_clear_sign_main,
        [
            "--signing-key",
            "test-signing-key",
            "--task-id",
            "1",
            "--config",
            f_config_msg_signer_ok,
            f"@{f_config_msg_signer_ok}",
        ],
    )
    assert result.exit_code == 0, result.output


def test__msg_clearsign_sign(f_msg_signer, f_config_msg_signer_ok):
    msg_clear_sign(
        ["hello world"], signing_key="test-signing-key", task_id="1", config=f_config_msg_signer_ok
    )

    f_msg_signer.return_value.load_config.assert_called_with(load_config(f_config_msg_signer_ok))
    operation = ClearSignOperation(
        inputs=["hello world"],
        signing_key="test-signing-key",
        task_id="1",
    )
    f_msg_signer.return_value.sign.assert_called_with(operation)


def test__msg_clearsign_sign_file_input(f_msg_signer, f_config_msg_signer_ok):
    msg_clear_sign(
        [f"@{f_config_msg_signer_ok}"],
        signing_key="test-signing-key",
        task_id="1",
        config=f_config_msg_signer_ok,
    )

    f_msg_signer.return_value.load_config.assert_called_with(load_config(f_config_msg_signer_ok))
    operation = ClearSignOperation(
        inputs=[open(f_config_msg_signer_ok).read()],
        signing_key="test-signing-key",
        task_id="1",
    )
    f_msg_signer.return_value.sign.assert_called_with(operation)


def test_get_config_file(f_config_msg_signer_ok):
    assert _get_config_file(f_config_msg_signer_ok) == f_config_msg_signer_ok


def test_get_config_file_default(f_config_msg_signer_ok):
    def _patched_exist(fname):
        if fname == "/etc/pubtools-sign/conf.yaml":
            return True
        else:
            return False

    with patch("os.path.exists", side_effect=_patched_exist):
        assert _get_config_file("/non-existining/file") == "/etc/pubtools-sign/conf.yaml"


def test_get_config_no_configuration_found():
    with patch("os.path.exists", return_value=False):
        with pytest.raises(ValueError):
            _get_config_file("/non-existining/file")


def test__construct_signing_message(f_config_msg_signer_ok):
    signer = MsgSigner()
    signer.load_config(load_config(f_config_msg_signer_ok))
    with patch("uuid.uuid4", return_value="1234-5678-abcd-efgh"):
        with patch("pubtools.sign.signers.msgsigner.isodate_now") as patched_date:
            patched_date.return_value = "created-date-Z"
            ret = signer._construct_signing_message("some-claim", "some-signing-key", {})
            assert ret == {
                "sig_key_id": "some-signing-key",
                "claim_file": "some-claim",
                "request_id": "1234-5678-abcd-efgh",
                "created": "created-date-Z",
                "requested_by": "pubtools-sign-test",
            }


def test__construct_headers(f_config_msg_signer_ok):
    signer = MsgSigner()
    signer.load_config(load_config(f_config_msg_signer_ok))
    with patch("uuid.uuid4", return_value="1234-5678-abcd-efgh"):
        with patch("pubtools.sign.signers.msgsigner.isodate_now") as patched_date:
            patched_date.return_value = "created-date-Z"
            ret = signer._construct_headers(
                SignRequestType.CONTAINER, extra_attrs={"extra": "extra"}
            )
            assert ret == {
                "service": "pubtools-sign",
                "environment": "prod",
                "owner_id": "pubtools-sign-test",
                "mtype": SignRequestType.CONTAINER,
                "source": "metadata",
                "extra": "extra",
            }


def test_operations():
    signer = MsgSigner()
    assert signer.operations() == [ContainerSignOperation, ClearSignOperation]


def test_create_msg_message(f_config_msg_signer_ok):
    signer = MsgSigner()
    signer.load_config(load_config(f_config_msg_signer_ok))

    data = "test-data"
    with patch("uuid.uuid4", return_value="1234-5678-abcd-efgh"):
        with patch("pubtools.sign.signers.msgsigner.isodate_now") as patched_date:
            patched_date.return_value = "created-date-Z"
            operation = ClearSignOperation(
                inputs=["test-data-inputs"], signing_key="test-key", task_id="1"
            )
            assert signer._create_msg_message(
                data, operation, SignRequestType.CONTAINER
            ) == MsgMessage(
                headers={
                    "service": "pubtools-sign",
                    "environment": "prod",
                    "owner_id": "pubtools-sign-test",
                    "mtype": SignRequestType.CONTAINER,
                    "source": "metadata",
                },
                address="topic://Topic.sign",
                body={
                    "sig_key_id": "test-key",
                    "claim_file": "test-data",
                    "request_id": "1234-5678-abcd-efgh",
                    "created": "created-date-Z",
                    "requested_by": "pubtools-sign-test",
                },
            )
            assert signer._create_msg_message(
                data, operation, SignRequestType.CLEARSIGN
            ) == MsgMessage(
                headers={
                    "service": "pubtools-sign",
                    "environment": "prod",
                    "owner_id": "pubtools-sign-test",
                    "mtype": SignRequestType.CLEARSIGN,
                    "source": "metadata",
                },
                address="topic://Topic.sign",
                body={
                    "sig_key_id": "test-key",
                    "data": "test-data",
                    "request_id": "1234-5678-abcd-efgh",
                    "created": "created-date-Z",
                    "requested_by": "pubtools-sign-test",
                },
            )


def test_sign(f_config_msg_signer_ok):
    signer = MsgSigner()
    signer.load_config(load_config(f_config_msg_signer_ok))
    container_sign_operation = ContainerSignOperation(
        digests=("some-digest",),
        references=("some-reference",),
        signing_key="test-signing-key",
        task_id="1",
    )
    clear_sign_operation = ClearSignOperation(
        inputs=["hello world"],
        signing_key="test-signing-key",
        task_id="1",
    )

    with patch("pubtools.sign.signers.msgsigner.MsgSigner.clear_sign") as patched_clear_sign:
        with patch(
            "pubtools.sign.signers.msgsigner.MsgSigner.container_sign"
        ) as patched_container_sign:
            signer.sign(container_sign_operation)
            patched_container_sign.assert_called_once()
            signer.sign(clear_sign_operation)
            patched_clear_sign.assert_called_once()
            with pytest.raises(UnsupportedOperation):
                signer.sign(Mock())


def test_create_manifest_claim_message():
    signer = MsgSigner()
    assert signer.create_manifest_claim_message(
        "some-key", "some-digest", "some-reference"
    ) == base64.b64encode(
        json.dumps(
            {
                "critical": {
                    "type": "atomic container signature",
                    "image": {"docker-manifest-digest": "some-digest"},
                    "identity": {"docker-reference": "some-reference"},
                },
                "optional": {"creator": "pubtools-sign"},
            }
        ).encode("latin-1")
    ).decode(
        "latin-1"
    )


@patch("uuid.uuid4", return_value="1234-5678-abcd-efgh")
def test_clear_sign(patched_uuid, f_config_msg_signer_ok):
    clear_sign_operation = ClearSignOperation(
        inputs=["hello world"],
        signing_key="test-signing-key",
        task_id="1",
    )
    with patch("pubtools.sign.signers.msgsigner.SendClient") as patched_send_client:
        with patch("pubtools.sign.signers.msgsigner.RecvClient") as patched_recv_client:
            patched_send_client.return_value.run.return_value = []
            patched_recv_client.return_value.run.return_value = []
            patched_recv_client.return_value.recv = {"1234-5678-abcd-efgh": "signed:'hello world'"}
            patched_recv_client.return_value.errors = []

            signer = MsgSigner()
            signer.load_config(load_config(f_config_msg_signer_ok))
            res = signer.clear_sign(clear_sign_operation)

            assert res == SigningResults(
                signer=signer,
                operation=clear_sign_operation,
                signer_results=MsgSignerResults(status="ok", error_message=""),
                operation_result=ClearSignResult(
                    outputs=["signed:'hello world'"], signing_key="test-signing-key"
                ),
            )


@patch("uuid.uuid4", return_value="1234-5678-abcd-efgh")
def test_clear_sign_recv_errors(patched_uuid, f_config_msg_signer_ok):
    clear_sign_operation = ClearSignOperation(
        inputs=["hello world"],
        signing_key="test-signing-key",
        task_id="1",
    )
    with patch("pubtools.sign.signers.msgsigner.SendClient") as patched_send_client:
        with patch("pubtools.sign.signers.msgsigner.RecvClient") as patched_recv_client:
            patched_send_client.return_value.run.return_value = []
            patched_recv_client.return_value.errors = [
                MsgError(
                    name="TestError", description="test error description", source="test-source"
                )
            ]
            patched_recv_client.return_value.recv = {"1234-5678-abcd-efgh": "signed:'hello world'"}

            signer = MsgSigner()
            signer.load_config(load_config(f_config_msg_signer_ok))
            res = signer.clear_sign(clear_sign_operation)

            assert res == SigningResults(
                signer=signer,
                operation=clear_sign_operation,
                signer_results=MsgSignerResults(
                    status="error", error_message="TestError : test error description\n"
                ),
                operation_result=ClearSignResult(outputs=[""], signing_key="test-signing-key"),
            )


@patch("uuid.uuid4", return_value="1234-5678-abcd-efgh")
def test_clear_sign_send_errors(patched_uuid, f_config_msg_signer_ok):
    clear_sign_operation = ClearSignOperation(
        inputs=["hello world"],
        signing_key="test-signing-key",
        task_id="1",
    )
    with patch("pubtools.sign.signers.msgsigner.SendClient") as patched_send_client:
        with patch("pubtools.sign.signers.msgsigner.RecvClient") as patched_recv_client:
            patched_send_client.return_value.run.return_value = [
                MsgError(
                    name="TestError", description="test error description", source="test-source"
                )
            ]
            patched_recv_client.return_value.run.return_value = []
            patched_recv_client.return_value.recv = {"1234-5678-abcd-efgh": "signed:'hello world'"}

            signer = MsgSigner()
            signer.load_config(load_config(f_config_msg_signer_ok))
            res = signer.clear_sign(clear_sign_operation)

            assert res == SigningResults(
                signer=signer,
                operation=clear_sign_operation,
                signer_results=MsgSignerResults(
                    status="error", error_message="TestError : test error description\n"
                ),
                operation_result=ClearSignResult(outputs=[""], signing_key="test-signing-key"),
            )


@patch("uuid.uuid4", return_value="1234-5678-abcd-efgh")
def test_container_sign(patched_uuid, f_config_msg_signer_ok, f_client_certificate):
    container_sign_operation = ContainerSignOperation(
        task_id="1",
        digests=["sha256:abcdefg"],
        references=["some-registry/namespace/repo:tag"],
        signing_key="test-signing-key",
    )

    with patch("pubtools.sign.signers.msgsigner.SendClient") as patched_send_client:
        with patch("pubtools.sign.signers.msgsigner.RecvClient") as patched_recv_client:
            patched_send_client.return_value.run.return_value = []
            patched_recv_client.return_value.run.return_value = []
            patched_recv_client.return_value.recv = {"1234-5678-abcd-efgh": "signed:'claim'"}
            patched_recv_client.return_value.errors = []

            signer = MsgSigner()
            signer.load_config(load_config(f_config_msg_signer_ok))
            res = signer.container_sign(container_sign_operation)

            patched_send_client.assert_called_with(
                messages=[ANY],
                broker_urls=["amqps://broker-01:5671", "amqps://broker-02:5671"],
                cert=f_client_certificate,
                ca_cert=os.path.expanduser("~/messaging/ca-cert.crt"),
                retries=3,
                errors=[],
            )

            assert res == SigningResults(
                signer=signer,
                operation=container_sign_operation,
                signer_results=MsgSignerResults(status="ok", error_message=""),
                operation_result=ContainerSignResult(
                    signed_claims=["signed:'claim'"], signing_key="test-signing-key"
                ),
            )


@patch("uuid.uuid4", return_value="1234-5678-abcd-efgh")
def test_container_sign_recv_errors(patched_uuid, f_config_msg_signer_ok):
    container_sign_operation = ContainerSignOperation(
        task_id="1",
        digests=["sha256:abcdefg"],
        references=["some-registry/namespace/repo:tag"],
        signing_key="test-signing-key",
    )

    with patch("pubtools.sign.signers.msgsigner.SendClient") as patched_send_client:
        with patch("pubtools.sign.signers.msgsigner.RecvClient") as patched_recv_client:
            patched_send_client.return_value.run.return_value = []
            patched_recv_client.return_value.errors = [
                MsgError(
                    name="TestError", description="test error description", source="test-source"
                )
            ]
            patched_recv_client.return_value.recv = {"1234-5678-abcd-efgh": "signed:'hello world'"}

            signer = MsgSigner()
            signer.load_config(load_config(f_config_msg_signer_ok))
            res = signer.container_sign(container_sign_operation)

            assert res == SigningResults(
                signer=signer,
                operation=container_sign_operation,
                signer_results=MsgSignerResults(
                    status="error", error_message="TestError : test error description\n"
                ),
                operation_result=ContainerSignResult(
                    signed_claims=[""], signing_key="test-signing-key"
                ),
            )


@patch("uuid.uuid4", return_value="1234-5678-abcd-efgh")
def test_container_sign_send_errors(patched_uuid, f_config_msg_signer_ok):
    container_sign_operation = ContainerSignOperation(
        task_id="1",
        digests=["sha256:abcdefg"],
        references=["some-registry/namespace/repo:tag"],
        signing_key="test-signing-key",
    )
    with patch("pubtools.sign.signers.msgsigner.SendClient") as patched_send_client:
        with patch("pubtools.sign.signers.msgsigner.RecvClient") as patched_recv_client:
            patched_send_client.return_value.run.return_value = [
                MsgError(
                    name="TestError", description="test error description", source="test-source"
                )
            ]
            patched_recv_client.return_value.run.return_value = []
            patched_recv_client.return_value.recv = {"1234-5678-abcd-efgh": "signed:'hello world'"}

            signer = MsgSigner()
            signer.load_config(load_config(f_config_msg_signer_ok))
            res = signer.container_sign(container_sign_operation)

            assert res == SigningResults(
                signer=signer,
                operation=container_sign_operation,
                signer_results=MsgSignerResults(
                    status="error", error_message="TestError : test error description\n"
                ),
                operation_result=ContainerSignResult(
                    signed_claims=[""], signing_key="test-signing-key"
                ),
            )


@patch("uuid.uuid4", return_value="1234-5678-abcd-efgh")
def test_container_sign_wrong_inputs(patched_uuid, f_config_msg_signer_ok):
    container_sign_operation = ContainerSignOperation(
        task_id="1",
        digests=["sha256:abcdefg"],
        references=["some-registry/namespace/repo:tag", "some-registry/namespace/repo:tag2"],
        signing_key="test-signing-key",
    )

    signer = MsgSigner()
    signer.load_config(load_config(f_config_msg_signer_ok))
    with pytest.raises(ValueError):
        signer.container_sign(container_sign_operation)


def test_msgsig_doc_arguments():
    assert MsgSigner.doc_arguments() == {
        "options": {
            "messaging_brokers": {"description": "List of brokers URLS"},
            "messaging_cert_key": {
                "description": "Client certificate + key for messaging authorization"
            },
            "messaging_ca_cert": {"description": "Messaging CA certificate"},
            "topic_send_to": {"description": "Topic where to send the messages"},
            "topic_listen_to": {"description": "Topic where to listen for replies"},
            "creator": {"description": "Identification of creator of signing request"},
            "environment": {"description": "Environment indetification in sent messages"},
            "service": {"description": "Service identificator"},
            "timeout": {"description": "Timeout for messaging sent/receive"},
            "retries": {"description": "Retries for messaging sent/receive"},
            "message_id_key": {
                "description": "Attribute name in message body which should be used as message id"
            },
            "log_level": {"description": "Log level"},
        },
        "examples": {
            "msg_signer": {
                "messaging_brokers": ["amqps://broker-01:5671", "amqps://broker-02:5671"],
                "messaging_cert_key": "~/messaging/cert.pem",
                "messaging_ca_cert": "~/messaging/ca_cert.crt",
                "topic_send_to": "topic://Topic.sign",
                "topic_listen_to": "queue://Consumer.{{creator}}.{{task_id}}.Topic.sign."
                "{{task_id}}",
                "creator": "pubtools-sign",
                "environment": "prod",
                "service": "pubtools-sign",
                "timeout": 1,
                "retries": 3,
                "message_id_key": "123",
                "log_level": "debug",
            }
        },
    }


def test_msgsigresult_to_dict():
    assert MsgSignerResults(status="status", error_message="error_message").to_dict() == {
        "status": "status",
        "error_message": "error_message",
    }


def test_msgsigresult_doc_arguments():
    assert MsgSignerResults.doc_arguments() == {
        "signer_result": {
            "type": "dict",
            "description": "Signing result status.",
            "returned": "always",
            "sample": {"status": "ok", "error_message": ""},
        }
    }



#!/usr/bin/python

from __future__ import absolute_import, division, print_function

from ansible.module_utils.basic import AnsibleModule

import yaml

from pubtools.sign.results import ClearSignResult
from pubtools.sign.operations.clearsign import ClearSignOperation
from pubtools.sign.signers.msgsigner import MsgSigner, msg_clear_sign, MsgSignerResults

__metaclass__ = type


DOCUMENTATION = r"""
---
module: msg_clear_sign
version_added: "0.1"
short_description: This is used to do clear sign
description:
    - Sign data with clear sign.
{0}  config:
    description:
      - Config file path.
      - By default, it will read from "~/.config/pubtools-sign/conf.yaml" or
        "/etc/pubtools-sign/conf.yaml"
    required: false
    type: str
{1}
extends_documentation_fragment:
    - action_common_attributes
attributes:
    check_mode:
        support: full
    diff_mode:
        support: none
    platform:
        platforms: posix
author:
    - zxiong (@redhat.com)
""".format(
    yaml.dump({"options": ClearSignOperation.doc_arguments().get("options")}),
    yaml.dump({"config file options": MsgSigner.doc_arguments().get("options")}),
)

EXAMPLES = r"""
# Example for msg clear sign
{0}
The example of the config file /etc/pubtools-sign/conf.yaml:
{1}
""".format(
    yaml.dump([{"msg_clear_sign": ClearSignOperation.doc_arguments().get("examples")}]),
    yaml.dump(MsgSigner.doc_arguments().get("examples")),
)

RETURN = r"""
# These are examples of possible return values, and in general should use other names for return
# values.
{0}
{1}
""".format(
    yaml.dump(MsgSignerResults.doc_arguments()), yaml.dump(ClearSignResult.doc_arguments())
)


def run_module():
    """Run clearsign module."""
    # Define available arguments/parameters a user can pass to the module
    module_args = dict(
        inputs=dict(type="list", required=True),
        signing_key=dict(type="str", required=True),
        task_id=dict(type="str", required=True),
        config=dict(type="str", required=False),
    )

    result = dict(changed=False, message="")
    module = AnsibleModule(argument_spec=module_args, supports_check_mode=True)

    # If the user is working with this module in only check mode we do not
    # want to make any changes to the environment, just return the current
    # state with no modifications
    if module.check_mode:
        module.exit_json(**result)

    inputs = module.params["inputs"]
    signing_key = module.params["signing_key"]
    task_id = module.params["task_id"]
    config = module.params["config"]

    # Call clear sign and return signed data
    try:
        signing_result = msg_clear_sign(
            inputs, signing_key=signing_key, task_id=task_id, config=config
        )
    except Exception as ex:
        module.fail_json(msg=str(ex), exception=ex)

    result["message"] = signing_result

    # signing failed
    if signing_result["signer_result"]["status"] != "ok":
        module.fail_json(msg=signing_result["signer_result"]["error_message"], **result)

    # signing successfully
    result["changed"] = True
    module.exit_json(**result)


if __name__ == "__main__":
    run_module()



from __future__ import annotations

import base64
from dataclasses import field, dataclass, asdict
import enum
import json
import logging
from typing import Dict, List, ClassVar, Any, Optional
import uuid
import os
import sys

import OpenSSL
import click

from . import Signer
from ..operations.base import SignOperation
from ..operations import ClearSignOperation, ContainerSignOperation
from ..results.signing_results import SigningResults
from ..results import ClearSignResult, ContainerSignResult
from ..results import SignerResults
from ..exceptions import UnsupportedOperation
from ..clients.msg_send_client import SendClient
from ..clients.msg_recv_client import RecvClient
from ..models.msg import MsgMessage
from ..conf.conf import load_config, CONFIG_PATHS
from ..utils import set_log_level, isodate_now


LOG = logging.getLogger("pubtools.sign.signers.msgsigner")


class SignRequestType(str, enum.Enum):
    """Sign request type enum."""

    CONTAINER = "container_signature"
    CLEARSIGN = "clearsign_signature"


@dataclass()
class MsgSignerResults(SignerResults):
    """MsgSignerResults model."""

    status: str
    error_message: str

    def to_dict(self: SignerResults):
        """Return dict representation of MsgSignerResults model."""
        return {"status": self.status, "error_message": self.error_message}

    @classmethod
    def doc_arguments(cls: SignerResults) -> Dict[str, Any]:
        """Return dictionary with result description of SignerResults."""
        doc_arguments = {
            "signer_result": {
                "type": "dict",
                "description": "Signing result status.",
                "returned": "always",
                "sample": {"status": "ok", "error_message": ""},
            }
        }

        return doc_arguments


@dataclass()
class MsgSigner(Signer):
    """Messaging signer class."""

    messaging_brokers: List[str] = field(
        init=False,
        metadata={
            "description": "List of brokers URLS",
            "sample": [
                "amqps://broker-01:5671",
                "amqps://broker-02:5671",
            ],
        },
    )
    messaging_cert_key: str = field(
        init=False,
        metadata={
            "description": "Client certificate + key for messaging authorization",
            "sample": "~/messaging/cert.pem",
        },
    )
    messaging_ca_cert: str = field(
        init=False,
        metadata={"description": "Messaging CA certificate", "sample": "~/messaging/ca_cert.crt"},
    )
    topic_send_to: str = field(
        init=False,
        metadata={
            "description": "Topic where to send the messages",
            "sample": "topic://Topic.sign",
        },
    )
    topic_listen_to: str = field(
        init=False,
        metadata={
            "description": "Topic where to listen for replies",
            "sample": "queue://Consumer.{{creator}}.{{task_id}}.Topic.sign.{{task_id}}",
        },
    )
    creator: str = field(
        init=False,
        metadata={
            "description": "Identification of creator of signing request",
            "sample": "pubtools-sign",
        },
    )
    environment: str = field(
        init=False,
        metadata={"description": "Environment indetification in sent messages", "sample": "prod"},
    )
    service: str = field(
        init=False, metadata={"description": "Service identificator", "sample": "pubtools-sign"}
    )
    timeout: int = field(
        init=False,
        default=60,
        metadata={"description": "Timeout for messaging sent/receive", "sample": 1},
    )
    retries: int = field(
        init=False,
        default=60,
        metadata={"description": "Retries for messaging sent/receive", "sample": 3},
    )
    message_id_key: str = field(
        init=False,
        metadata={
            "description": "Attribute name in message body which should be used as message id",
            "sample": "123",
        },
    )
    log_level: str = field(init=False, metadata={"description": "Log level", "sample": "debug"})

    SUPPORTED_OPERATIONS: ClassVar[List[SignOperation]] = [
        ContainerSignOperation,
        ClearSignOperation,
    ]

    def _construct_signing_message(
        self: MsgSigner,
        claim,
        signing_key,
        extra_attrs: Optional[Dict] = None,
        sig_type: SignRequestType = SignRequestType.CONTAINER,
    ):
        data_attr = "claim_file" if sig_type == SignRequestType.CONTAINER else "data"
        _extra_attrs = extra_attrs or {}
        message = {
            "sig_key_id": signing_key,
            data_attr: claim,
            "request_id": str(uuid.uuid4()),
            "created": isodate_now(),
            "requested_by": self.creator,
        }
        message.update(_extra_attrs)
        return message

    def _construct_headers(self: MsgSigner, sig_type, extra_attrs: Optional[Dict] = None):
        headers = {
            "service": self.service,
            "environment": self.environment,
            "owner_id": self.creator,
            "mtype": sig_type.value,
            "source": "metadata",
        }
        if extra_attrs:
            headers.update(extra_attrs)
        return headers

    def _create_msg_message(
        self: MsgSigner, data, operation: SignOperation, sig_type: SignRequestType, extra_attrs=None
    ):
        ret = MsgMessage(
            headers=self._construct_headers(sig_type, extra_attrs=extra_attrs),
            body=self._construct_signing_message(
                data, operation.signing_key, extra_attrs=extra_attrs, sig_type=sig_type.value
            ),
            address=self.topic_send_to.format(
                **dict(list(asdict(self).items()) + list(asdict(operation).items()))
            ),
        )
        LOG.debug(f"Construted message with request_id {ret.body['request_id']}")
        return ret

    def load_config(self: MsgSigner, config_data: Dict[str, Any]) -> None:
        """Load configuration of messaging signer."""
        self.messaging_brokers = config_data["msg_signer"]["messaging_brokers"]
        self.messaging_cert_key = os.path.expanduser(
            config_data["msg_signer"]["messaging_cert_key"]
        )
        self.messaging_ca_cert = os.path.expanduser(config_data["msg_signer"]["messaging_ca_cert"])
        self.topic_send_to = config_data["msg_signer"]["topic_send_to"]
        self.topic_listen_to = config_data["msg_signer"]["topic_listen_to"]
        self.environment = config_data["msg_signer"]["environment"]
        self.service = config_data["msg_signer"]["service"]
        self.message_id_key = config_data["msg_signer"]["message_id_key"]
        self.retries = config_data["msg_signer"]["retries"]
        self.log_level = config_data["msg_signer"]["log_level"]
        self.timeout = config_data["msg_signer"]["timeout"]
        self.creator = self._get_cert_subject_cn()

    def _get_cert_subject_cn(self):
        x509 = OpenSSL.crypto.load_certificate(
            OpenSSL.crypto.FILETYPE_PEM, open(os.path.expanduser(self.messaging_cert_key)).read()
        )
        return x509.get_subject().CN

    def operations(self: MsgSigner) -> List[SignOperation]:
        """Return list of supported operations."""
        return self.SUPPORTED_OPERATIONS

    def sign(self: MsgSigner, operation: SignOperation) -> SigningResults:
        """Run signing operation.

        :param operation: signing operation
        :type operation: SignOperation

        :return: SigningResults
        """
        if isinstance(operation, ClearSignOperation):
            return self.clear_sign(operation)
        elif isinstance(operation, ContainerSignOperation):
            return self.container_sign(operation)
        else:
            raise UnsupportedOperation(operation)

    def clear_sign(self: MsgSigner, operation: ClearSignOperation):
        """Run the clearsign operation.

        :param operation: signing operation
        :type operation: ClearSignOperation

        :return: SigningResults
        """
        set_log_level(LOG, self.log_level)
        messages = []
        message_to_data = {}
        for in_data in operation.inputs:
            message = self._create_msg_message(
                base64.b64encode(in_data.encode("latin1")).decode("latin-1"),
                operation,
                SignRequestType.CLEARSIGN,
                extra_attrs={"pub_task_id": operation.task_id},
            )
            message_to_data[message.body["request_id"]] = message
            messages.append(message)

        signer_results = MsgSignerResults(status="ok", error_message="")
        operation_result = ClearSignResult(
            signing_key=operation.signing_key, outputs=[""] * len(operation.inputs)
        )
        signing_results = SigningResults(
            signer=self,
            operation=operation,
            signer_results=signer_results,
            operation_result=operation_result,
        )
        LOG.debug(f"{len(messages)} messages to send")

        errors = []
        errors = SendClient(
            messages=messages,
            broker_urls=self.messaging_brokers,
            cert=self.messaging_cert_key,
            ca_cert=self.messaging_ca_cert,
            retries=self.retries,
            errors=errors,
        ).run()

        if errors:
            signer_results.status = "error"
            for error in errors:
                signer_results.error_message += f"{error.name} : {error.description}\n"
            return signing_results

        message_ids = [message.body["request_id"] for message in messages]

        recvc = RecvClient(
            message_ids=message_ids,
            topic=self.topic_listen_to.format(
                **dict(list(asdict(self).items()) + list(asdict(operation).items()))
            ),
            id_key=self.message_id_key,
            broker_urls=self.messaging_brokers,
            cert=self.messaging_cert_key,
            ca_cert=self.messaging_ca_cert,
            timeout=self.timeout,
            retries=self.retries,
            errors=errors,
        )
        recvc.run()
        errors = recvc.errors
        if errors:
            signer_results.status = "error"
            for error in errors:
                signer_results.error_message += f"{error.name} : {error.description}\n"
            return signing_results

        operation_result = ClearSignResult(
            signing_key=operation.signing_key, outputs=[""] * len(messages)
        )
        for recv_id, received in recvc.recv.items():
            operation_result.outputs[messages.index(message_to_data[recv_id])] = received
        signing_results.operation_result = operation_result
        return signing_results

    @staticmethod
    def create_manifest_claim_message(signature_key, digest, reference):
        """Create manifest claim for container signing.

        See below for the specification for the manifest claim that is created here
        https://github.com/containers/image/blob/master/docs/atomic-signature.md
        """
        manifest_claim = {
            "critical": {
                "type": "atomic container signature",
                "image": {"docker-manifest-digest": digest},
                "identity": {"docker-reference": reference},
            },
            "optional": {"creator": "pubtools-sign"},
        }
        return base64.b64encode(json.dumps(manifest_claim).encode("latin1")).decode("latin1")

    def container_sign(self: MsgSigner, operation: ContainerSignOperation):
        """Run container signing operation.

        :param operation: signing operation
        :type operation: ContainerSignOperation

        :return: SigningResults
        """
        set_log_level(LOG, self.log_level)
        messages = []
        message_to_data = {}
        if len(operation.digests) != len(operation.references):
            raise ValueError("Digests must pairs with references")

        for digest, reference in zip(operation.digests, operation.references):
            message = self._create_msg_message(
                self.create_manifest_claim_message(
                    operation.signing_key, digest=digest, reference=reference
                ),
                operation,
                SignRequestType.CONTAINER,
                extra_attrs={"pub_task_id": operation.task_id},
            )
            message_to_data[message.body["request_id"]] = message
            messages.append(message)

        signer_results = MsgSignerResults(status="ok", error_message="")
        operation_result = ContainerSignResult(
            signing_key=operation.signing_key, signed_claims=[""] * len(operation.digests)
        )
        signing_results = SigningResults(
            signer=self,
            operation=operation,
            signer_results=signer_results,
            operation_result=operation_result,
        )
        LOG.debug(f"{len(messages)} messages to send")

        errors = []
        errors = SendClient(
            messages=messages,
            broker_urls=self.messaging_brokers,
            cert=self.messaging_cert_key,
            ca_cert=self.messaging_ca_cert,
            retries=self.retries,
            errors=errors,
        ).run()

        if errors:
            signer_results.status = "error"
            for error in errors:
                signer_results.error_message += f"{error.name} : {error.description}\n"
            return signing_results

        message_ids = [message.body["request_id"] for message in messages]

        recvc = RecvClient(
            message_ids=message_ids,
            topic=self.topic_listen_to.format(
                **dict(list(asdict(self).items()) + list(asdict(operation).items()))
            ),
            id_key=self.message_id_key,
            broker_urls=self.messaging_brokers,
            cert=self.messaging_cert_key,
            ca_cert=self.messaging_ca_cert,
            timeout=self.timeout,
            retries=self.retries,
            errors=errors,
        )
        recvc.run()
        errors = recvc.errors

        if recvc.errors:
            signer_results.status = "error"
            for error in errors:
                signer_results.error_message += f"{error.name} : {error.description}\n"
            return signing_results

        operation_result = ContainerSignResult(
            signing_key=operation.signing_key, signed_claims=[""] * len(messages)
        )
        for recv_id, received in recvc.recv.items():
            operation_result.signed_claims[messages.index(message_to_data[recv_id])] = received
        signing_results.operation_result = operation_result
        return signing_results


def _get_config_file(config_candidate):
    if not os.path.exists(config_candidate):
        for config_candidate in CONFIG_PATHS:
            if os.path.exists(os.path.expanduser(config_candidate)):
                break
        else:
            raise ValueError(
                "No configuration file found: %s" % list(set(CONFIG_PATHS + [config_candidate]))
            )
    return config_candidate


def msg_clear_sign(inputs, signing_key=None, task_id=None, config=""):
    """Run clearsign operation."""
    msg_signer = MsgSigner()
    config = _get_config_file(config)
    msg_signer.load_config(load_config(os.path.expanduser(config)))

    str_inputs = []
    for input_ in inputs:
        if input_.startswith("@"):
            str_inputs.append(open(input_.lstrip("@")).read())
        else:
            str_inputs.append(input_)
    operation = ClearSignOperation(inputs=str_inputs, signing_key=signing_key, task_id=task_id)
    signing_result = msg_signer.sign(operation)
    return {
        "signer_result": signing_result.signer_results.to_dict(),
        "operation_results": signing_result.operation_result.outputs,
        "signing_key": signing_result.operation_result.signing_key,
    }


def msg_container_sign(signing_key=None, task_id=None, config="", digest=None, reference=None):
    """Run containersign operation with cli arguments."""
    msg_signer = MsgSigner()
    config = _get_config_file(config)
    msg_signer.load_config(load_config(os.path.expanduser(config)))

    operation = ContainerSignOperation(
        digests=digest, references=reference, signing_key=signing_key, task_id=task_id
    )
    signing_result = msg_signer.sign(operation)
    return {
        "signer_result": signing_result.signer_results.to_dict(),
        "operation_results": signing_result.operation_result.signed_claims,
        "signing_key": signing_result.operation_result.signing_key,
    }


@click.command()
@click.option(
    "--signing-key",
    required=True,
    help="8 characters key fingerprint of key which should be used for signing",
)
@click.option("--task-id", required=True, help="Task id identifier (usually pub task-id)")
@click.option("--config", default=CONFIG_PATHS[0], help="path to the config file")
@click.option("--raw", default=False, is_flag=True, help="Print raw output instead of json")
@click.argument("inputs", nargs=-1)
def msg_clear_sign_main(inputs, signing_key=None, task_id=None, config=None, raw=None):
    """Entry point method for clearsign operation."""
    ret = msg_clear_sign(inputs, signing_key=signing_key, task_id=task_id, config=config)
    if not raw:
        click.echo(json.dumps(ret))
        print(ret)
        if ret["signer_result"]["status"] == "error":
            sys.exit(1)
    else:
        if ret["signer_result"]["status"] == "error":
            print(ret["signer_result"]["error_message"], file=sys.stderr)
            sys.exit(1)
        else:
            for claim in ret["operation_results"]:
                print(claim)


@click.command()
@click.option(
    "--signing-key",
    required=True,
    help="8 characters key fingerprint of key which should be used for signing",
)
@click.option("--task-id", required=True, help="Task id identifier (usually pub task-id)")
@click.option("--config", default=CONFIG_PATHS[0], help="path to the config file")
@click.option(
    "--digest",
    required=True,
    multiple=True,
    type=str,
    help="Digests which should be signed.",
)
@click.option(
    "--reference",
    required=True,
    multiple=True,
    type=str,
    help="References which should be signed.",
)
@click.option("--raw", default=False, is_flag=True, help="Print raw output instead of json")
def msg_container_sign_main(
    signing_key=None, task_id=None, config=None, digest=None, reference=None, raw=None
):
    """Entry point method for containersign operation."""
    ret = msg_container_sign(
        signing_key=signing_key,
        task_id=task_id,
        config=config,
        digest=digest,
        reference=reference,
    )
    if not raw:
        click.echo(json.dumps(ret))
        if ret["signer_result"]["status"] == "error":
            sys.exit(1)
    else:
        if ret["signer_result"]["status"] == "error":
            print(ret["signer_result"]["error_message"], file=sys.stderr)
            sys.exit(1)
        else:
            for claim in ret["operation_results"]:
                print(claim)



from __future__ import annotations

from abc import ABC, abstractmethod
import dataclasses

from .clearsign import ClearSignResult  # noqa: F401
from .containersign import ContainerSignResult  # noqa: F401


@dataclasses.dataclass()
class SignerResults(ABC):
    """SignerResults abstract class."""

    @abstractmethod
    def to_dict(self: SignerResults):
        """Return dict representation of SignerResults."""
        ...  # pragma: no cover



from typing import List
from typing_extensions import Self

from .operation_result import OperationResult

from typing import Dict, Any

import dataclasses


@dataclasses.dataclass()
class ClearSignResult(OperationResult):
    """ClearOperationResult model."""

    outputs: List[str]
    signing_key: str

    def to_dict(self: Self):
        """Return dict representation of ClearOperationResult."""
        return {"outputs": self.outputs, "signing_key": self.signing_key}

    @classmethod
    def doc_arguments(cls: OperationResult) -> Dict[str, Any]:
        """Return dictionary with arguments description of the operation."""
        doc_arguments = {
            "operation_results": {
                "type": "list",
                "description": "Signing result output",
                "returned": "always",
                "sample": ["signed:'hello world'"],
            },
            "signing_key": {
                "type": "str",
                "description": "The signing key which is used during signing.",
                "returned": "always",
                "sample": "123",
            },
        }

        return doc_arguments



import click


from .signers.msgsigner import msg_clear_sign_main, msg_container_sign_main


@click.group()
def cli():
    """Pubtools-sign multi command bundle."""
    pass


cli.add_command(msg_clear_sign_main, name="msg-clear-sign")
cli.add_command(msg_container_sign_main, name="msg-container-sign")



from unittest.mock import patch
import time
from threading import Thread

from pubtools.sign.clients.msg_send_client import SendClient, _SendClient
from pubtools.sign.clients.msg_recv_client import RecvClient, _RecvClient
from pubtools.sign.models.msg import MsgMessage


def test_recv_client_zero_messages(
    f_cleanup_msgsigner_messages,
    f_qpid_broker,
    f_fake_msgsigner,
    f_msgsigner_send_to_queue,
    f_client_certificate,
    f_ca_certificate,
):
    qpid_broker, port = f_qpid_broker
    errors = []
    rc = RecvClient(
        f_msgsigner_send_to_queue,
        [],
        [f"localhost:{port}"],
        "request_id",
        f_client_certificate,
        f_ca_certificate,
        1.0,
        1,
        errors,
    )
    rc.run()
    msgsigner, _, received_messages = f_fake_msgsigner
    assert [x.body for x in msgsigner.received_messages] == []


def test_recv_client_recv_message(
    f_cleanup_msgsigner_messages,
    f_qpid_broker,
    f_msgsigner_listen_to_topic,
    f_fake_msgsigner,
    f_msgsigner_send_to_queue,
):
    qpid_broker, port = f_qpid_broker
    message = MsgMessage(
        headers={"mtype": "test"},
        address=f_msgsigner_listen_to_topic,
        body={"msg": {"message": "test_message", "request_id": "1"}},
    )

    sender = SendClient([message], [f"localhost:{port}"], "", "", 10, [])
    errors = []
    receiver = RecvClient(
        f_msgsigner_send_to_queue,
        ["1"],
        "request_id",
        [f"localhost:{port}"],
        "",
        "",
        60.0,
        2,
        errors,
    )

    tsc = Thread(target=sender.run, args=())
    trc = Thread(target=receiver.run, args=())

    trc.start()
    tsc.start()

    time.sleep(1)

    assert receiver.recv == {
        "1": ({"msg": {"message": "test_message", "request_id": "1"}}, {"mtype": "test"})
    }

    receiver.stop()
    sender.stop()
    tsc.join()
    trc.join()


def test_recv_client_timeout(
    f_cleanup_msgsigner_messages,
    f_qpid_broker,
    f_msgsigner_listen_to_topic,
    f_fake_msgsigner,
    f_msgsigner_send_to_queue,
):
    qpid_broker, port = f_qpid_broker
    message = MsgMessage(
        headers={"mtype": "test"},
        address=f_msgsigner_listen_to_topic,
        body={"msg": {"message": "test_message", "request_id": "1"}},
    )
    errors = []
    sender = SendClient([message], [f"localhost:{port}"], "", "", 10, [])
    receiver = RecvClient(
        f_msgsigner_send_to_queue + "_wrong",
        ["1"],
        "request_id",
        [f"localhost:{port}"],
        "",
        "",
        10.0,
        1,
        errors,
    )

    tsc = Thread(target=sender.run, args=())
    trc = Thread(target=receiver.run, args=())

    trc.start()
    tsc.start()

    time.sleep(1)

    assert receiver.recv == {}


def test_recv_client_transport_error(
    f_cleanup_msgsigner_messages,
    f_qpid_broker,
    f_msgsigner_listen_to_topic,
    f_fake_msgsigner,
    f_msgsigner_send_to_queue,
):
    qpid_broker, port = f_qpid_broker
    errors = []
    receiver = RecvClient(
        f_msgsigner_send_to_queue,
        ["1"],
        "request_id",
        [f"localhost:{port+1}"],
        "",
        "",
        10.0,
        1,
        errors,
    )

    trc = Thread(target=receiver.run, args=())

    trc.start()

    time.sleep(1)
    assert len(errors) == 1


def test_recv_client_link_error(
    f_cleanup_msgsigner_messages,
    f_broken_qpid_broker,
    f_msgsigner_listen_to_topic,
    f_fake_msgsigner,
    f_msgsigner_send_to_queue,
):
    qpid_broker, port = f_broken_qpid_broker
    message = MsgMessage(
        headers={"mtype": "test"},
        address=f_msgsigner_listen_to_topic,
        body={"msg": {"message": "test_message", "request_id": "1"}},
    )
    sender = SendClient([message], [f"localhostx:{port}"], "", "", 10, [])
    errors = []
    receiver = RecvClient(
        f_msgsigner_send_to_queue,
        ["1"],
        "request_id",
        [f"localhost:{port}"],
        "",
        "",
        10.0,
        1,
        errors,
    )

    tsc = Thread(target=sender.run, args=())
    trc = Thread(target=receiver.run, args=())

    trc.start()
    tsc.start()
    time.sleep(1)
    assert len(errors) == 1


def test_recv_client_errors(
    f_cleanup_msgsigner_messages,
    f_qpid_broker,
    f_msgsigner_listen_to_topic,
    f_fake_msgsigner,
    f_msgsigner_send_to_queue,
):
    qpid_broker, port = f_qpid_broker
    message = MsgMessage(
        headers={"mtype": "test"},
        address=f_msgsigner_listen_to_topic,
        body={"msg": {"message": "test_message", "request_id": "1"}},
    )
    sender = SendClient([message], [f"localhostx:{port}"], "", "", 10, [])
    errors = []

    on_message_original = _RecvClient.on_message
    with patch(
        "pubtools.sign.clients.msg_recv_client._RecvClient.on_message", autospec=True
    ) as patched_on_message:
        patched_on_message.side_effect = lambda self, event: [
            self.errors.append("1"),
            on_message_original(self, event),
        ]

        receiver = RecvClient(
            f_msgsigner_send_to_queue,
            ["1"],
            "request_id",
            [f"localhost:{port}"],
            "",
            "",
            10.0,
            1,
            errors,
        )

        tsc = Thread(target=sender.run, args=())
        trc = Thread(target=receiver.run, args=())

        trc.start()
        tsc.start()
        time.sleep(1)
        assert len(errors) == 1


def test_recv_client_recv_message_stray(
    f_cleanup_msgsigner_messages,
    f_qpid_broker,
    f_msgsigner_listen_to_topic_stray,
    f_msgsigner_send_to_queue_stray,
    f_fake_msgsigner_stray,
):
    qpid_broker, port = f_qpid_broker
    message = MsgMessage(
        headers={"mtype": "test"},
        address=f_msgsigner_listen_to_topic_stray,
        body={"msg": {"message": "test_message", "request_id": "1"}},
    )

    sender = SendClient([message], [f"localhost:{port}"], "", "", 10, [])
    errors = []
    receiver = RecvClient(
        f_msgsigner_send_to_queue_stray,
        ["1"],
        "request_id",
        [f"localhost:{port}"],
        "",
        "",
        60.0,
        2,
        errors,
    )

    tsc = Thread(target=sender.run, args=())
    trc = Thread(target=receiver.run, args=())

    trc.start()
    tsc.start()

    time.sleep(1)

    assert receiver.recv == {}

    receiver.stop()
    sender.stop()
    tsc.join()
    trc.join()


def test_recv_client_recv_message_timeout(
    f_cleanup_msgsigner_messages,
    f_qpid_broker,
    f_msgsigner_listen_to_topic,
    f_fake_msgsigner,
    f_msgsigner_send_to_queue,
):
    qpid_broker, port = f_qpid_broker
    message = MsgMessage(
        headers={"mtype": "test"},
        address=f_msgsigner_listen_to_topic,
        body={"msg": {"message": "test_message", "request_id": "1"}},
    )

    on_sendable_original = _SendClient.on_sendable
    with patch(
        "pubtools.sign.clients.msg_send_client._SendClient.on_sendable", autospec=True
    ) as patched_on_sendable:
        patched_on_sendable.side_effect = lambda self, event: [
            on_sendable_original(self, event),
            time.sleep(10),
        ]

        sender = SendClient([message], [f"localhost:{port}"], "", "", 10, [])
        errors = []
        receiver = RecvClient(
            f_msgsigner_send_to_queue,
            ["1"],
            "request_id",
            [f"localhost:{port}"],
            "",
            "",
            1.0,
            2,
            errors,
        )

        tsc = Thread(target=sender.run, args=())
        trc = Thread(target=receiver.run, args=())

        trc.start()
        tsc.start()

        time.sleep(5)

        receiver.stop()
        sender.stop()
        tsc.join()
        trc.join()



from __future__ import annotations

import dataclasses

from typing import ForwardRef

from pubtools.sign.operations.base import SignOperation
from pubtools.sign.results import SignerResults
from pubtools.sign.results.operation_result import OperationResult


Signer = ForwardRef("Signer")


@dataclasses.dataclass
class SigningResults:
    """SigningResults model."""

    signer: Signer
    operation: SignOperation
    signer_results: SignerResults
    operation_result: OperationResult



import json
import logging

from ..models.msg import MsgError

from .msg import _MsgClient

import proton
import proton.utils
from proton.reactor import Container


LOG = logging.getLogger("pubtools.sign.client.msg_recv_client")


class _RecvClient(_MsgClient):
    def __init__(
        self, topic, message_ids, id_key, broker_urls, cert, ca_cert, timeout, recv, errors
    ):
        super().__init__(errors=errors)
        self.broker_urls = broker_urls
        self.topic = topic
        self.id_key = id_key
        self.ssl_domain = proton.SSLDomain(proton.SSLDomain.MODE_CLIENT)
        if cert:
            self.ssl_domain.set_credentials(cert, cert, None)
        if ca_cert:
            self.ssl_domain.set_trusted_ca_db(ca_cert)
        self.ssl_domain.set_peer_authentication(proton.SSLDomain.ANONYMOUS_PEER)
        self.recv_ids = {x: False for x in message_ids}
        self.confirmed = 0
        self.recv = recv
        self.timeout = timeout

    def on_start(self, event):
        LOG.debug("RECEIVER: On start %s %s %s", event, self.topic, self.broker_urls)
        self.timer_task = event.container.schedule(self.timeout, self)
        conn = event.container.connect(
            urls=self.broker_urls, ssl_domain=self.ssl_domain, sasl_enabled=False
        )
        self.receiver = event.container.create_receiver(conn, self.topic)
        self.timer_task = event.container.schedule(self.timeout, self)

    def on_message(self, event):
        LOG.debug("RECEIVER: On message (%s)", event)
        outer_message = json.loads(event.message.body)
        headers = event.message.properties
        msg_id = outer_message["msg"][self.id_key]

        if msg_id in self.recv_ids:
            self.recv_ids[msg_id] = True
            self.recv[msg_id] = (outer_message, headers)
            self.accept(event.delivery)
        else:
            LOG.debug(f"RECEIVER: Ignored message {msg_id}")

        if self.recv_ids.values() and all(self.recv_ids.values()):
            self.timer_task.cancel()
            event.receiver.close()
            event.connection.close()

    def on_timer_task(self, event):
        LOG.debug("RECEIVER: On timeout (%s)", event)
        self.timer_task.cancel()
        if event.connection:
            event.connection.close()  # pragma: no cover
        if event.receiver:
            event.receiver.close()  # pragma: no cover

        self.errors.append(
            MsgError(
                source=event,
                name="MessagingTimeout",
                description="Out of time when receiving messages",
            )
        )


class RecvClient(Container):
    """Messaging receiver."""

    def __init__(
        self, topic, message_ids, id_key, broker_urls, cert, ca_cert, timeout, retries, errors
    ):
        """Recv Client Initializer.

        :param topic: Topic where to listen for incoming messages (for example topic://Topic.signed)
        :type topic: str
        :param message_ids: List of awaited message ids
        :type topic: List[str]
        :param id_key: Attribute name in message body which is considered as id
        :type topic: str
        :param message_ids: List of broker urls
        :type topic: List[str]
        :param cert: Messaging client certificate
        :type cert: str
        :param ca_cert: Messaging ca certificate
        :type ca_cert: str
        :param timeout: Timeout for the messaging receiver
        :type timeout: int
        :param retries: How many attempts to retry receiving messages
        :type retries: int
        :param errors: List of errors which occured during the process
        :type errors: List[MsgError]
        """
        self.message_ids = message_ids
        self.recv = {}
        self._errors = errors
        self.handler = _RecvClient(
            topic=topic,
            message_ids=message_ids,
            id_key=id_key,
            broker_urls=broker_urls,
            cert=cert,
            ca_cert=ca_cert,
            timeout=timeout,
            recv=self.recv,
            errors=self._errors,
        )
        self._retries = retries
        super().__init__(self.handler)

    def run(self):
        """Run the receiver."""
        errors_len = 0
        if not len(self.message_ids):
            LOG.warning("No messages to receive")
            return []

        for x in range(self._retries):
            super().run()
            if len(self._errors) == errors_len:
                break
            errors_len = self._errors
        else:
            return self._errors
        return self.recv



from __future__ import annotations
from abc import ABC, abstractmethod
import dataclasses
from typing import Dict, List, Any

from ..results.signing_results import SigningResults
from ..operations.base import SignOperation


@dataclasses.dataclass
class Signer(ABC):
    """Abstract Signer class."""

    @abstractmethod
    def load_config(self: Signer, config_data: Dict[str, Any]) -> None:
        """Load configuration to the Signer.

        :param config_data: Dict with configuration data.
        :type config_data: Dict[str, Any]
        """
        ...  # pragma: no cover

    @abstractmethod
    def operations(self: Signer) -> List[SignOperation]:
        """Return list of operations supported by the Signer.

        :return: List[SignOperation]
        """
        ...  # pragma: no cover

    @abstractmethod
    def sign(self: Signer, operation: SignOperation) -> SigningResults:
        """Run signing operation.

        :param operation: signing operation
        :type operation: SignOperation

        :return: SigningResults
        """
        ...  # pragma: no cover

    @classmethod
    def doc_arguments(cls: Signer) -> Dict[str, Any]:
        """Return dict with arguments decription for the signer.

        :return: Dict[str, Any]
        """
        doc_arguments = {}
        options_arguments_doc = {}
        exmaple_arguments_doc = {}

        for fn, fv in cls.__dataclass_fields__.items():
            if fv.metadata.get("description"):
                options_arguments_doc[fn] = {
                    field: fv.metadata[field] for field in fv.metadata if field != "sample"
                }
                exmaple_arguments_doc[fn] = fv.metadata.get("sample", "")

        doc_arguments["options"] = options_arguments_doc
        doc_arguments["examples"] = {"msg_signer": exmaple_arguments_doc}

        return doc_arguments



from unittest.mock import Mock


from pubtools.sign.clients.msg import _MsgClient
from pubtools.sign.models.msg import MsgError


def test_msg_handler_errors():
    mock_error = Mock()

    errors = []
    msgsc = _MsgClient(errors)
    msgsc.on_link_error(mock_error)
    assert errors == [
        MsgError(name=mock_error, description=mock_error.link.condition, source=mock_error.link)
    ]

    errors = []
    msgsc = _MsgClient(errors)
    msgsc.on_session_error(mock_error)
    assert errors == [
        MsgError(
            name=mock_error, description=mock_error.session.condition, source=mock_error.session
        )
    ]

    errors = []
    msgsc = _MsgClient(errors)
    msgsc.on_connection_error(mock_error)
    assert errors == [
        MsgError(
            name=mock_error,
            description=mock_error.connection.condition,
            source=mock_error.connection,
        )
    ]
